"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.QueryAutoCompleter = void 0;
const autocompleter_1 = require("../autocompleter");
const util_1 = require("./util");
class QueryAutoCompleter {
    constructor(version, textCompleter, fields) {
        this.version = version;
        this.textCompleter = textCompleter;
        this.fields = fields;
        this.getCompletions = (editor, session, position, prefix, done) => {
            if (prefix === '')
                return done(null, []);
            const currentToken = session.getTokenAt(position.row, position.column);
            if ((currentToken === null || currentToken === void 0 ? void 0 : currentToken.type) === 'string') {
                return this.textCompleter.getCompletions(editor, session, position, prefix, done);
            }
            done(null, (0, autocompleter_1.completer)(prefix, {
                serverVersion: this.version,
                fields: (0, util_1.getNames)(this.fields),
                meta: ['query', 'bson', 'field:identifier'],
            }).map((completion) => {
                if (completion.meta === 'field:identifier') {
                    return {
                        ...completion,
                        value: (0, autocompleter_1.wrapField)(completion.value),
                    };
                }
                return completion;
            }));
        };
    }
    update(fields) {
        this.fields = fields;
    }
}
exports.QueryAutoCompleter = QueryAutoCompleter;
//# sourceMappingURL=query-autocompleter.js.map