"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.StageAutoCompleter = void 0;
const query_autocompleter_1 = require("./query-autocompleter");
const autocompleter_1 = require("../autocompleter");
const util_1 = require("./util");
const PROJECT = '$project';
const GROUP = '$group';
const MATCH = '$match';
const DOLLAR = '$';
class StageAutoCompleter {
    constructor(version, textCompleter, fields, stageOperator = null) {
        this.version = version;
        this.textCompleter = textCompleter;
        this.fields = fields;
        this.stageOperator = stageOperator;
        this.getCompletions = (editor, session, position, prefix, done) => {
            var _a;
            if (prefix === '') {
                return done(null, []);
            }
            const currentToken = session.getTokenAt(position.row, position.column);
            if (!currentToken) {
                return done(null, []);
            }
            if (currentToken.type === 'string') {
                if (prefix.startsWith(DOLLAR)) {
                    return done(null, (0, autocompleter_1.completer)(prefix, {
                        fields: (0, util_1.getNames)(this.fields),
                        meta: ['field:reference'],
                    }));
                }
                return this.textCompleter.getCompletions(editor, session, position, prefix, done);
            }
            if (currentToken === null || currentToken === void 0 ? void 0 : currentToken.type.includes('comment')) {
                return done(null, []);
            }
            if (this.stageOperator && this.stageOperator === MATCH) {
                this.queryAutoCompleter.getCompletions(editor, session, position, prefix, done);
            }
            else {
                return done(null, (0, autocompleter_1.completer)(prefix, {
                    serverVersion: this.version,
                    fields: (0, util_1.getNames)(this.fields),
                    meta: [
                        'expr:*',
                        'conv',
                        'bson',
                        'field:identifier',
                        ...([PROJECT, GROUP].includes((_a = this.stageOperator) !== null && _a !== void 0 ? _a : '')
                            ? ['accumulator', 'accumulator:*']
                            : []),
                    ],
                }));
            }
        };
        this.queryAutoCompleter = new query_autocompleter_1.QueryAutoCompleter(version, textCompleter, fields);
    }
    update(fields, stageOperator, severVersion) {
        this.fields = fields;
        this.queryAutoCompleter.update(fields);
        this.stageOperator = stageOperator;
        this.version = severVersion !== null && severVersion !== void 0 ? severVersion : this.version;
    }
    updateStageOperator(stageOperator) {
        this.stageOperator = stageOperator;
    }
}
exports.StageAutoCompleter = StageAutoCompleter;
//# sourceMappingURL=stage-autocompleter.js.map