import React from 'react';
import type { KeyBinding } from '@codemirror/view';
import { EditorView } from '@codemirror/view';
import type { Diagnostic } from '@codemirror/lint';
import type { CompletionSource } from '@codemirror/autocomplete';
import type { IconGlyph } from '@mongodb-js/compass-components';
import { Facet } from '@codemirror/state';
import { LanguageSupport } from '@codemirror/language';
type EditorLanguage = 'json' | 'javascript' | 'javascript-expression';
export type Annotation = Pick<Diagnostic, 'from' | 'to' | 'severity' | 'message'>;
type EditorProps = {
    language?: EditorLanguage;
    onChangeText?: (text: string, event?: any) => void;
    onLoad?: (editor: EditorView) => void;
    darkMode?: boolean;
    showLineNumbers?: boolean;
    showFoldGutter?: boolean;
    showAnnotationsGutter?: boolean;
    showScroll?: boolean;
    highlightActiveLine?: boolean;
    readOnly?: boolean;
    'data-testid'?: string;
    annotations?: Annotation[];
    completer?: CompletionSource;
    minLines?: number;
    maxLines?: number;
    lineHeight?: number;
    placeholder?: string;
    commands?: readonly KeyBinding[];
    initialJSONFoldAll?: boolean;
} & ({
    text: string;
    initialText?: never;
} | {
    text?: never;
    initialText: string;
}) & Pick<React.HTMLProps<HTMLDivElement>, 'id' | 'className' | 'onFocus' | 'onBlur'>;
export declare const languages: Record<EditorLanguage, () => LanguageSupport>;
export declare const languageName: Facet<EditorLanguage, readonly EditorLanguage[]>;
export type EditorRef = {
    foldAll: () => boolean;
    unfoldAll: () => boolean;
    copyAll: () => boolean;
    prettify: () => boolean;
    applySnippet: (template: string) => boolean;
    focus: () => boolean;
    readonly editor: EditorView | null;
};
declare const BaseEditor: React.ForwardRefExoticComponent<EditorProps & React.RefAttributes<EditorRef>>;
export type { EditorView, KeyBinding as Command };
type InlineEditorProps = Omit<EditorProps, 'text' | 'showLineNumbers' | 'showFoldGutter' | 'showAnnotationsGutter' | 'showScroll' | 'highlightActiveLine' | 'minLines' | 'annotations'> & ({
    text: string;
    initialText?: never;
} | {
    text?: never;
    initialText: string;
});
declare const InlineEditor: React.ForwardRefExoticComponent<InlineEditorProps & React.RefAttributes<EditorRef>>;
export type Action = {
    icon: IconGlyph;
    label: string;
    action: (editor: EditorView) => boolean | void;
};
type MultilineEditorProps = EditorProps & {
    customActions?: Action[];
    copyable?: boolean;
    formattable?: boolean;
    editorClassName?: string;
    actionsClassName?: string;
};
declare const MultilineEditor: React.ForwardRefExoticComponent<MultilineEditorProps & React.RefAttributes<EditorRef>>;
declare function setCodemirrorEditorValue(element: HTMLElement | string | null, text: string): Promise<void>;
export { BaseEditor };
export { InlineEditor as CodemirrorInlineEditor };
export { MultilineEditor as CodemirrorMultilineEditor };
export { setCodemirrorEditorValue };
export type { CompletionSource as Completer };
//# sourceMappingURL=json-editor.d.ts.map